module IssueFromMessage
  module Patches
    module IssuesControllerPatch
      def self.included(base) # :nodoc:
        base.send(:include, InstanceMethods)

        base.class_eval do
          unloadable

	        alias_method_chain :create, :ifm
	        alias_method_chain :new, :ifm
        end
      end

      module ClassMethods
      end

      module InstanceMethods
	      def new_with_ifm
		      @ifm_message=retrieve_message_from_id
		      if @ifm_message
						@issue.subject=ifm_build_issue_subject(@ifm_message) if !@issue.subject || @issue.subject.size==0
			      @issue.start_date=Date.parse(@ifm_message.created_on.to_s)
						@issue.assigned_to=User.current unless @issue.assigned_to
			      if !@issue.description || @issue.description.size==0
							@issue.description=(params[:ifm_quote_text].blank? ? '' : @ifm_message.content)
							@issue.description="message##{@ifm_message.id}\n\n"+@issue.description unless params[:ifm_message_link].blank?
							@issue.description="project:#{@ifm_message.project.identifier}\n\n"+@issue.description unless params[:ifm_project_link].blank?
						end
		      end
		      new_without_ifm
	      end

        # code taken from IssuesController::create!!
	      def create_with_ifm
# VVK start
		      @ifm_message=retrieve_message_from_id
		      if !@ifm_message || params[:continue_to_issues]
			      params.delete(:continue_to_issues)
			      create_without_ifm
			      return
		      end
# VVK end

		      call_hook(:controller_issues_new_before_save, { :params => params, :issue => @issue })
		      if @issue.save
		        attachments = Attachment.attach_files(@issue, params[:attachments])
		        call_hook(:controller_issues_new_after_save, { :params => params, :issue => @issue})
		        respond_to do |format|
		          format.html {
		            render_attachment_warning_if_needed(@issue)
		            flash[:notice] = l(:notice_issue_successful_create, :id => "<a href='#{issue_path(@issue)}'>##{@issue.id}</a>")
# VVK start
		            redirect_to(params[:continue].blank? ? create_msg_url_hash(@ifm_message)  : { :action => 'show', :id => @issue })
# VVK end
		          }
		          format.api  { render :action => 'show', :status => :created, :location => issue_url(@issue) }
		        end
		        return
		      else
		        respond_to do |format|
# VVK start
		          format.html { render :action => 'new', :ifm_message_id => @ifm_message.id,
		                               :ifm_quote_text => params[:ifm_quote_text],
		                               :ifm_message_link => params[:ifm_message_link],
		                               :ifm_project_link => params[:ifm_project_link],
		                               :ifm_prefix_text => params[:ifm_prefix_text]
		          }
# VVK end
		          format.api  { render_validation_errors(@issue) }
		        end
		      end
	      end

private

	      def create_msg_url_hash(msg)
		      link_to_str = { :controller => 'messages', :action => 'show', :board_id => msg.board }
		      if msg.root && msg.root != msg
			      link_to_str[:id]=msg.root
			      link_to_str[:r]=msg
			      link_to_str[:anchor]="message-#{msg.id}"
		      else
			      link_to_str[:id]=msg
		      end
		      link_to_str
	      end

	      def retrieve_message_from_id
		      return nil if params[:ifm_message_id].blank?

					msg=Message.find_by_id(params[:ifm_message_id].to_i)
					return (msg && msg.visible?) ? msg : nil
	      end

	      def ifm_build_issue_subject(msg)
		      subj=Mailer.create_message_posted(msg).subject
		      subj="#{params[:ifm_prefix_text]}  #{subj}" unless params[:ifm_prefix_text].blank?
		      return subj
	      end
      end
    end
  end
end
